using System;
using System.Collections.Generic;
using System.Text;

namespace ShapeBlending
{
    public class Blending
    {
        /// <summary>
        /// Blends shapes together connected in a surface rectangle.
        /// </summary>
        /// <param name="basePoints">The standard baseshape.</param>
        /// <param name="statePoints">The different state of points.</param>
        /// <param name="blendFactors">The blending control factors.</param>
        /// <returns>Returns the blended points.</returns>
        public static Point3f[,] Blend(Point3f[,] basePoints, 
            Point3f[][,] statePoints, float[] blendFactors)
        {
            int w = basePoints.GetLength(0);
            int h = basePoints.GetLength(1);
            int numShapes = statePoints.Length;
            float sumFactors = 0;
            Point3f[,] ps = new Point3f[w, h];
            for (int x = 0; x < w; x++)
                for (int y = 0; y < h; y++)
                    ps[x, y] = new Point3f(0, 0, 0);
            float oldSumFactors = 1;
            float newSumFactors = 0;
            float blendFactor = 0;
            Point3f[,] sh = null;
            for (int i = 0; i < numShapes; i++)
            {
                sh = statePoints[i];
                blendFactor = blendFactors[i];
                newSumFactors = Math.Max(1, sumFactors + blendFactor);
                // Blend points.
                for (int x = 0; x < w; x++)
                    for (int y = 0; y < h; y++)
                        ps[x, y] = (ps[x, y] * (oldSumFactors / newSumFactors)) +
                            ((sh[x, y] - basePoints[x, y]) * blendFactor);
                sumFactors += blendFactor;
                oldSumFactors = Math.Max(1, sumFactors);
            }
            for (int x = 0; x < w; x++)
                for (int y = 0; y < h; y++)
                    ps[x, y] = basePoints[x, y] + ps[x, y];

            return ps;
        }

        /// <summary>
        /// Blends shapes into a new shape.
        /// </summary>
        /// <param name="baseVertices">The base polygons.</param>
        /// <param name="stateVertices">The different states of polygons.</param>
        /// <param name="blendFactors">The blending factors.</param>
        /// <returns>Returns the</returns>
        public static float[][] Blend(float[][] baseVertices,
            float[][][] stateVertices, float[] blendFactors)
        {
            int numVertices = baseVertices.Length;
            int numShapes = stateVertices.Length;
            float sumFactors = 0;
            float[][] ps = new float[numVertices][];
            // Set all points to origo.
            for (int i = 0; i < numVertices; i++)
                ps[i] = new float[] { 0, 0, 0 };

            float oldSumFactors = 1;
            float newSumFactors = 0;
            float blendFactor = 0;
            float[][] sh = null;
            for (int i = 0; i < numShapes; i++)
            {
                sh = stateVertices[i];
                blendFactor = blendFactors[i];
                newSumFactors = Math.Max(1, sumFactors + blendFactor);
                // Blend points.
                for (int j = 0; j < numVertices; j++ )
                    ps[j] = new float[]{
                        (ps[j][0] * (oldSumFactors / newSumFactors)) + (sh[j][0] - baseVertices[j][0]) * blendFactor,
                        (ps[j][1] * (oldSumFactors / newSumFactors)) + (sh[j][1] - baseVertices[j][1]) * blendFactor,
                        (ps[j][2] * (oldSumFactors / newSumFactors)) + (sh[j][2] - baseVertices[j][2]) * blendFactor
                    };
                sumFactors += blendFactor;
                oldSumFactors = Math.Max(1, sumFactors);
            }
            for (int j = 0; j < numVertices; j++)
                ps[j] = new float[]{
                    baseVertices[j][0] + ps[j][0],
                    baseVertices[j][1] + ps[j][1],
                    baseVertices[j][2] + ps[j][2]};

            return ps;
        }

        public static int[] GetChangedIndices(float[][] baseVertices,
            float[][][] stateVertices, float movement)
        {
            bool changed;
            int m = 0;
            int[] changes = new int[baseVertices.Length];

            for (int i = 0; i < baseVertices.Length; i++)
            {
                changed = false;

                // See if some of the state vertices changes
                // from the base vertices.
                for (int j = 0; j < stateVertices.Length; j++)
                {
                    if (Math.Abs(baseVertices[i][0] - stateVertices[j][i][0]) > movement)
                    {
                        changed = true;
                        break;
                    }
                    if (Math.Abs(baseVertices[i][1] - stateVertices[j][i][1]) > movement)
                    {
                        changed = true;
                        break;
                    }
                    if (Math.Abs(baseVertices[i][2] - stateVertices[j][i][2]) > movement)
                    {
                        changed = true;
                        break;
                    }
                }
                if (changed)
                {
                    changes[m] = i;
                    m++;
                }
            }

            // Copy over to a new array with right size.
            int[] newChanges = new int[m];
            for (int i = 0; i < m; i++)
                newChanges[i] = changes[i];
            return newChanges;
        }

        /// <summary>
        /// Blends shapes with a given array
        /// of which indices to change.
        /// </summary>
        /// <param name="baseVertices">The base shape of polygons.</param>
        /// <param name="stateVertices">The different states of polygons.</param>
        /// <param name="blendFactors">The blend factors.</param>
        /// <param name="changeIndices">The changed indices.</param>
        /// <returns>Returns the blended shape.</returns>
        public static float[][] Blend(float[][] baseVertices,
            float[][][] stateVertices, float[] blendFactors, int[] changeIndices)
        {
            int numVertices = baseVertices.Length;
            int numShapes = stateVertices.Length;
            float sumFactors = 0;
            float[][] ps = new float[numVertices][];
            // Set all points to origo.
            for (int i = 0; i < numVertices; i++)
                ps[i] = new float[] { 0, 0, 0 };

            float oldSumFactors = 1;
            float newSumFactors = 0;
            float blendFactor = 0;
            float[][] sh = null;
            int changeIndex = 0;

            for (int i = 0; i < numShapes; i++)
            {
                sh = stateVertices[i];
                blendFactor = blendFactors[i];
                newSumFactors = Math.Max(1, sumFactors + blendFactor);
                // Blend points.
                for (int j = 0; j < changeIndices.Length; j++)
                {
                    changeIndex = changeIndices[j];
                    ps[changeIndex] = new float[]{
                        (ps[changeIndex][0] * (oldSumFactors / newSumFactors)) + (sh[changeIndex][0] - baseVertices[changeIndex][0]) * blendFactor,
                        (ps[changeIndex][1] * (oldSumFactors / newSumFactors)) + (sh[changeIndex][1] - baseVertices[changeIndex][1]) * blendFactor,
                        (ps[changeIndex][2] * (oldSumFactors / newSumFactors)) + (sh[changeIndex][2] - baseVertices[changeIndex][2]) * blendFactor
                    };
                }
                sumFactors += blendFactor;
                oldSumFactors = Math.Max(1, sumFactors);
            }
            for (int j = 0; j < numVertices; j++)
                ps[j] = new float[]{
                    baseVertices[j][0] + ps[j][0],
                    baseVertices[j][1] + ps[j][1],
                    baseVertices[j][2] + ps[j][2]};

            return ps;
        }
    }
}
