using System;
using System.Collections.Generic;
using System.Text;
using Tao.OpenGl;

namespace ShapeBlending
{
    class CameraNavigation
    {
        private Point3f m_position = new Point3f(3, 0, 0);
        private Point3f m_upVector = new Point3f(0, 0, 1);
        private Point3f m_rightVector = new Point3f(1, 0, 0);
        private Point3f m_target = new Point3f(0, 0, 0);

        /// <summary>
        /// Creates a new camera.
        /// </summary>
        public CameraNavigation()
        {
            // Normalize the right vector.
            m_rightVector = Point3f.Normalize(Point3f.CrossProduct(m_target - m_position, m_upVector));
        }

        /// <summary>
        /// Gets or sets the position.
        /// </summary>
        public Point3f Position
        {
            get
            {return m_position;}
            set
            {
                m_position = value;

                // Normalize the right vector.
                Point3f viewDirection = Point3f.Normalize(m_target - m_position);
                m_rightVector = Point3f.Normalize(
                    Point3f.CrossProduct(viewDirection, m_upVector));
                // Normalize up vector.
                m_upVector = Point3f.CrossProduct(viewDirection, m_rightVector) * -1;
            }
        }

        public Point3f Target
        {
            get
            { return m_target; }
            set
            {
                m_target = value;
                Point3f viewDirection = Point3f.Normalize(m_target - m_position);
                // Normalize the right vector.
                m_rightVector = Point3f.Normalize(
                    Point3f.CrossProduct(viewDirection, m_upVector));
                // Normalize up vector.
                m_upVector = Point3f.CrossProduct(viewDirection, m_rightVector) * -1;
            }
        }

        public Point3f RightVector
        {
            get
            { return m_rightVector; }
        }

        /// <summary>
        /// Gets or sets the target point.
        /// </summary>

        /// <summary>
        /// Gets or sets the up vector.
        /// </summary>
        public Point3f UpVector
        {
            get
            { return m_upVector; }
            set
            { m_upVector = value; }
        }

        /// <summary>
        /// Sets up camera in OpenGL.
        /// </summary>
        public void SetUpCamera()
        {
            Gl.glLoadIdentity();
            Glu.gluLookAt(m_position.X, m_position.Y, m_position.Z,
                m_target.X, m_target.Y, m_target.Z,
                m_upVector.X, m_upVector.Y, m_upVector.Z);
        }

        public void Yaw(float degrees)
        { 
            float radians = (float)(degrees * Math.PI / 180);
            Point3f viewDirection = Point3f.Normalize(Point3f.Normalize(m_target - m_position) * (float)Math.Cos(radians)
                + m_upVector * (float)Math.Sin(radians));
            m_target = viewDirection * Point3f.Length(m_target - m_position);
            m_upVector = Point3f.CrossProduct(Point3f.Normalize(m_target - m_position), m_rightVector) * -1;
        }

        public void Pitch(float degrees)
        {
            float radians = (float)(degrees * Math.PI / 180);

            Point3f viewDirection = Point3f.Normalize(Point3f.Normalize(m_target - m_position) * (float)Math.Cos(radians)
                - m_rightVector * (float)Math.Sin(radians));
            m_target = viewDirection * Point3f.Length(m_target - m_position);
            m_rightVector = Point3f.CrossProduct(viewDirection, m_upVector);
        }

        public void Roll(float degrees)
        {
            float radians = (float)(degrees * Math.PI / 180);
            m_rightVector = Point3f.Normalize(m_rightVector * (float)Math.Cos(radians)
                + m_upVector * (float)Math.Sin(radians));
            m_upVector = Point3f.CrossProduct(Point3f.Normalize(m_target - m_position), m_rightVector) * -1;
        }

        public void MoveForward(float distance)
        {
            m_position += Point3f.Normalize(m_target - m_position) * distance;
        }

        public void Move(Point3f direction, float distance)
        {
            direction = Point3f.Normalize(direction);
            m_position += direction * distance;
            m_target += direction * distance;
        }

        public void StrafeRight(float distance)
        {
            m_target += m_rightVector * distance;
            m_position += m_rightVector * distance;
        }

        public void MoveUpward(float distance)
        {
            m_position += m_upVector * distance;
            m_target += m_upVector * distance;
        }

        public void Rotate(Point3f axis, float degrees)
        {
            float radians = (float)(degrees * Math.PI / 180);
            Point3f viewDirection = Point3f.Normalize(m_target - m_position);
            Matrix4f rotateMat = new Matrix4f();
            rotateMat.SetRotationAxis(degrees, new Vector3f(axis.X, axis.Y, axis.Z));
            Vector3f v1 = new Vector3f(m_upVector.X, m_upVector.Y, m_upVector.Z);
            Vector3f v2 = new Vector3f(m_rightVector.X, m_rightVector.Y, m_rightVector.Z);
            Vector3f v3 = new Vector3f(viewDirection.X, viewDirection.Y, viewDirection.Z);
            v1 = rotateMat.GetRotatedVector3f(v1);
            v2 = rotateMat.GetRotatedVector3f(v2);
            v3 = rotateMat.GetRotatedVector3f(v3);
            m_upVector = new Point3f(v1.X, v1.Y, v1.Z);
            m_rightVector = new Point3f(v2.X, v2.Y, v2.Z);
            viewDirection = new Point3f(v3.X, v3.Y, v3.Z);
            m_target = m_position + viewDirection * Point3f.Length(m_target - m_position);
        }

        public void RotateAround(Point3f axis, float degrees)
        {
            float radians = (float)(degrees * Math.PI / 180);
            Vector3f d = new Vector3f(m_position.X - m_target.X,
                m_position.Y - m_target.Y,
                m_position.Z - m_target.Z);
            Matrix4f rotateMat = new Matrix4f();
            rotateMat.SetRotationAxis(degrees, new Vector3f(axis.X, axis.Y, axis.Z));
            Vector3f v1 = new Vector3f(m_upVector.X, m_upVector.Y, m_upVector.Z) + d;
            Vector3f v2 = new Vector3f(m_rightVector.X, m_rightVector.Y, m_rightVector.Z) + d;
            v1 = rotateMat.GetRotatedVector3f(v1);
            v2 = rotateMat.GetRotatedVector3f(v2);
            d = rotateMat.GetRotatedVector3f(d);
            m_position = m_target + new Point3f(d.X, d.Y, d.Z);
            m_upVector = new Point3f(v1.X - d.X, v1.Y - d.Y, v1.Z - d.Z);
            m_rightVector = new Point3f(v2.X - d.X, v2.Y - d.Y, v2.Z - d.Z);
        }
    }
}
