using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Drawing;
using Tao.OpenGl;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace ShapeBlending
{
    /// <summary>
    /// Controls drawing and interacting with controls
    /// in 3D.
    /// </summary>
    class ControlManager
    {
        public const int HANDLE_OFFSET = 4000;

        private ArrayList m_controls = new ArrayList();
        private ArrayList m_buffers = new ArrayList();

        private SimpleControl m_capturingControl = null;

        private GlutWindow m_window;

        public ControlManager(GlutWindow window)
        {
            m_window = window;
            window.MouseMove += new MouseEventHandler(MouseMove);
        }

        public void MouseMove(object sender, MouseEventArgs e)
        {
            if (m_capturingControl == null)
            {
                for (int i = m_controls.Count - 1; i >= 0; i--)
                {
                    SimpleControl control = (SimpleControl)m_controls[i];
                    if (e.X >= control.Position.X && e.Y >= control.Position.Y &&
                        e.X <= control.Position.X + control.Size.Width &&
                        e.Y <= control.Position.Y + control.Size.Height)
                    {
                        // As control whether it wants to capture mouse.
                        if (control.CaptureMouse(e.X, e.Y, m_window))
                        {
                            m_capturingControl = control;
                            break;
                        }
                    }
                }
            }
            else if (m_capturingControl.ReleaseMouse(e.X, e.Y))
            {
                // Mouse is released.
                m_capturingControl = null;
            }
        }

        /// <summary>
        /// Adds a new control to the control manager.
        /// </summary>
        /// <param name="control">The new control to add.</param>
        public void AddControl(SimpleControl control)
        {
            m_controls.Add(control);
            Bitmap bmp = new Bitmap(BufferSize(control.Size.Width), 
                BufferSize(control.Size.Height));
            m_buffers.Add(bmp);
        }

        /// <summary>
        /// Computes the buffer size if image is to small.
        /// </summary>
        /// <param name="size">The size of image.</param>
        /// <returns>The buffer size.</returns>
        private int BufferSize(int size)
        {
            return (int)(Math.Pow(2, Math.Ceiling(Math.Log(size) / Math.Log(2))));
        }

        public void Draw()
        {
            // Turn off depth test while drawing.
            float one = 2.0f / (float)m_window.Width;
            float two = -2.0f / (float)m_window.Height;
            float three = -((float)m_window.Width / 2.0f);
            float four = -((float)m_window.Height / 2.0f);

            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glPushMatrix();
            Gl.glLoadIdentity();

            Gl.glScalef(one, two, 1.0f);
            Gl.glTranslatef(three, four, 0.0f); 

            Gl.glDisable(Gl.GL_DEPTH_TEST);
            Gl.glEnable(Gl.GL_TEXTURE_2D);

            // Enable alpha blend on pictures.
            Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);
            Gl.glEnable(Gl.GL_BLEND);

            for (int i = 0; i < m_controls.Count; i++)
            {
                SimpleControl control = (SimpleControl)m_controls[i];
                Bitmap buffer = (Bitmap)m_buffers[i];
                int handle = 0;
                Graphics g;
                if (buffer.Width != BufferSize(control.Size.Width) ||
                    buffer.Height != BufferSize(control.Size.Height))
                {
                    // Size has changed. 
                    // Create a new bitmap.
                    buffer.Dispose();
                    buffer = new Bitmap(BufferSize(control.Size.Width), 
                        BufferSize(control.Size.Height));
                }

                BitmapData data = buffer.LockBits(new Rectangle(0, 0, buffer.Width, buffer.Height),
                     ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
                Gl.glGenTextures(1, out handle);
                Gl.glBindTexture(Gl.GL_TEXTURE_2D, handle);
                Gl.glTexImage2D(Gl.GL_TEXTURE_2D, 0, Gl.GL_RGBA8,
                    buffer.Width, buffer.Height, 0, Gl.GL_BGRA, Gl.GL_UNSIGNED_BYTE, data.Scan0);
                Gl.glTexParameteri(Gl.GL_TEXTURE_2D, Gl.GL_TEXTURE_MIN_FILTER, Gl.GL_LINEAR);
                Gl.glTexParameteri(Gl.GL_TEXTURE_2D, Gl.GL_TEXTURE_MAG_FILTER, Gl.GL_LINEAR);
                buffer.UnlockBits(data);
                Gl.glBindTexture(Gl.GL_TEXTURE_2D, handle);

                // Draw control.
                g = Graphics.FromImage(buffer);
                g.Clear(Color.Transparent);
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                control.Draw(g);

                float scaleX = ((float)control.Size.Width) / buffer.Width;
                float scaleY = ((float)control.Size.Height) / buffer.Height;

                Gl.glBegin(Gl.GL_QUADS);
                Gl.glTexParameteri(Gl.GL_TEXTURE_2D, Gl.GL_TEXTURE_WRAP_S, Gl.GL_CLAMP);
                Gl.glTexParameteri(Gl.GL_TEXTURE_2D, Gl.GL_TEXTURE_WRAP_T, Gl.GL_CLAMP);

                Gl.glTexCoord2f(0.0f, 0.0f);
                Gl.glColor3f(1.0f, 1.0f, 1.0f);
                Gl.glVertex2i(control.Position.X, control.Position.Y);

                Gl.glTexCoord2f(scaleX, 0.0f);
                Gl.glVertex2i(control.Position.X + control.Size.Width, control.Position.Y);

                Gl.glTexCoord2f(scaleX, scaleY);
                Gl.glVertex2i(control.Position.X + control.Size.Width, control.Position.Y + control.Size.Height);

                Gl.glTexCoord2f(0.0f, scaleY);
                Gl.glVertex2i(control.Position.X, control.Position.Y + control.Size.Height);

                Gl.glEnd();

                Gl.glDeleteTextures(1, ref handle);
            }

            Gl.glDisable(Gl.GL_BLEND);

            Gl.glDisable(Gl.GL_TEXTURE_2D);
            Gl.glEnable(Gl.GL_DEPTH_TEST);

            Gl.glPopMatrix();
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
        }
    }
}
