using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Tao.OpenGl;
using Tao.FreeGlut;
using System.Drawing;
using System.Threading;

namespace ShapeBlending
{
    public class GlutWindow
    {

        private string m_title = "Glut Window";
        private bool m_running = true;
        private int m_width = 640;
        private int m_height = 480;
        private int m_xpos = 100;
        private int m_ypos = 100;
        private PointF m_cursorPos;
        private bool[] m_keystate = new bool[256];
        private bool[] m_mousestate = new bool[6];
        // The interval of timer in milliseconds.
        private int m_interval = 50;

        private bool m_draw = true;

        /// <summary>
        /// Gets or sets the width.
        /// </summary>
        public int Width
        {
            get { return m_width; }
            set { m_width = value; }
        }

        /// <summary>
        /// Gets or sets the height.
        /// </summary>
        public int Height
        {
            get { return m_height; }
            set { m_height = value; }
        }

        /// <summary>
        /// Gets the state of each mouse.
        /// </summary>
        public bool[] MouseState
        {
            get { return m_mousestate; }
        }

        /// <summary>
        /// Gets the position of cursor.
        /// </summary>
        public PointF CursorPos
        {
            get { return m_cursorPos; }
        }

        public string Title
        {
            get { return m_title; }
            set { m_title = value; }
        }

        public event EventHandler SetUpLights;

        public event EventHandler Display;

        public event EventHandler Reshape;

        public event KeyEventHandler KeyDown;

        public event KeyEventHandler KeyUp;

        public event EventHandler LoadTextures;

        public event MouseEventHandler MouseDown;

        public event MouseEventHandler MouseUp;

        public event EventHandler WindowClosing;

        public event MouseEventHandler MouseMove;

        /// <summary>
        /// Runs the shape blending program.
        /// </summary>
        public void Run()
        { 
            Glut.glutInit();
            Glut.glutInitDisplayMode(Glut.GLUT_RGB | Glut.GLUT_SINGLE);
            Glut.glutInitWindowSize(m_width, m_height);
            Glut.glutInitWindowPosition(m_xpos, m_ypos);
            Glut.glutCreateWindow(m_title);
            Gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

            // Set up lights.
            ProcessSetUpLights();
            ProcessReshape(m_width, m_height);

            Glut.glutDisplayFunc(new Glut.DisplayCallback(ProcessDisplay));
            Glut.glutReshapeFunc(new Glut.ReshapeCallback(ProcessReshape));
            Glut.glutKeyboardFunc(new Glut.KeyboardCallback(ProcessKeyDown));
            Glut.glutKeyboardUpFunc(new Glut.KeyboardUpCallback(ProsessKeyUp));
            Glut.glutMouseFunc(new Glut.MouseCallback(ProcessMouseFunc));
            Glut.glutCloseFunc(new Glut.CloseCallback(ProcessWindowClosing));
            Glut.glutMotionFunc(new Glut.MotionCallback(ProcessMouseMove));
            Glut.glutPassiveMotionFunc(new Glut.PassiveMotionCallback(ProcessMouseMove));

            // Tell to load textures.
            ProcessLoadTextures();

            // Run while running.
            while (m_running)
            {
                Thread.Sleep(m_interval);
                Application.DoEvents();
                if (m_draw)
                    ProcessDisplay();
            }

            if (WindowClosing != null)
                WindowClosing(this, new EventArgs());
        }

        /// <summary>
        /// Fires the init event.
        /// </summary>
        public void ProcessSetUpLights()
        {
            if (SetUpLights != null)
            {
                SetUpLights(this, new EventArgs());
            }
        }

        /// <summary>
        /// Fires the display event.
        /// </summary>
        public void ProcessDisplay()
        {
            if (Display != null)
                Display(this, new EventArgs());
        }

        /// <summary>
        /// Reshapes the graphics to fit to window.
        /// </summary>
        /// <param name="w">The new width of window.</param>
        /// <param name="h">The new height of window.</param>
        public void ProcessReshape(int w, int h)
        {
            m_draw = false;
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            Gl.glViewport(0, 0, w, h);
            Glu.gluPerspective(30.0f, (double)w / h, 0.1f, 70.0f);
            Gl.glMatrixMode(Gl.GL_MODELVIEW);

            m_width = w;
            m_height = h;
            if (Reshape != null)
                Reshape(this, new EventArgs());
            m_draw = true;
        }

        public void ProcessKeyDown(byte key, int x, int y)
        {
            m_cursorPos = new PointF(x, y);
            m_keystate[key] = true;
            if (KeyDown != null)
                KeyDown(this, new KeyEventArgs(KeyCodeMap.GetKey(key)));
        }

        public void ProsessKeyUp(byte key, int x, int y)
        {
            m_cursorPos = new PointF(x, y);
            m_keystate[key] = false;
            if (KeyUp != null)
                KeyUp(this, new KeyEventArgs(KeyCodeMap.GetKey(key)));
        }

        /// <summary>
        /// Is called when user changes the state of a mouse button.
        /// </summary>
        /// <param name="button">The button that was pressed or released.</param>
        /// <param name="state">The state of button.</param>
        /// <param name="x">The x-position of cursor.</param>
        /// <param name="y">The y-position of cursor.</param>
        public void ProcessMouseFunc(int button, int state, int x, int y)
        {
            m_cursorPos = new PointF(x, y);
            if (button < m_mousestate.Length)
            {
                m_mousestate[button] = (state == 0 ? true : false);
                MouseButtons b = MouseButtons.None;
                switch (button)
                {
                    case 0:
                        b = MouseButtons.Left;
                        break;
                    case 1:
                        b = MouseButtons.Right;
                        break;
                    case 2:
                        b = MouseButtons.Middle;
                        break;
                    case 3:
                        b = MouseButtons.XButton1;
                        break;
                    case 4:
                        b = MouseButtons.XButton2;
                        break;
                }

                // Raise mouse event.
                if (m_mousestate[button] && MouseDown != null)
                    MouseDown(this, new MouseEventArgs(b, 1, x, y, 0));
                else if (!m_mousestate[button] && MouseUp != null)
                    MouseUp(this, new MouseEventArgs(b, 1, x, y, 0));
            }
        }

        public void ProcessLoadTextures()
        {
            if (LoadTextures != null)
                LoadTextures(this, new EventArgs());
        }

        /// <summary>
        /// Process the window closing event.
        /// </summary>
        public void ProcessWindowClosing()
        {
            m_draw = false;
            m_running = false;
        }

        /// <summary>
        /// Handles mouse movement.
        /// </summary>
        /// <param name="x">The x position of mouse.</param>
        /// <param name="y">The y position of mouse.</param>
        public void ProcessMouseMove(int x, int y)
        {
            if (MouseMove != null)
                MouseMove(this, new MouseEventArgs(MouseButtons.None, 0, x, y, 0));
        }

        /// <summary>
        /// Closes the application.
        /// </summary>
        public void Close()
        {
            m_running = false;
        }

        /// <summary>
        /// Returns true if a specific mouse button is down.
        /// </summary>
        /// <param name="button">The mouse button to check.</param>
        /// <returns>Returns true if the mouse button is down.</returns>
        public bool IsMouseDown(MouseButtons button)
        {
            switch (button)
            { 
                case MouseButtons.Left:
                    return m_mousestate[0];
                case MouseButtons.Right:
                    return m_mousestate[1];
                case MouseButtons.Middle:
                    return m_mousestate[2];
                case MouseButtons.XButton1:
                    return m_mousestate[3];
                case MouseButtons.XButton2:
                    return m_mousestate[4];
            }
            return false;
        }
    }
}