using System;
using System.Collections.Generic;
using System.Text;

namespace ShapeBlending
{
    /// <summary>
    /// Contains a 3D point.
    /// v1.1
    /// </summary>
    public struct Point3f
    {
        public float X;
        public float Y;
        public float Z;

        public static Point3f EMPTY = new Point3f(float.NaN, float.NaN, float.NaN);

        /// <summary>
        /// Creates a new 3D point.
        /// </summary>
        /// <param name="x">The x-value of point.</param>
        /// <param name="y">The y-value of point.</param>
        /// <param name="z">The z-value of point.</param>
        public Point3f(float x, float y, float z)
        {
            this.X = x;
            this.Y = y;
            this.Z = z;
        }

        /// <summary>
        /// Operator for adding two 3D points.
        /// </summary>
        /// <param name="p1">The first point to add.</param>
        /// <param name="p2">The second point to add.</param>
        /// <returns></returns>
        public static Point3f operator + (Point3f p1, Point3f p2)
        {
            return new Point3f(p1.X + p2.X, p1.Y + p2.Y, p1.Z + p2.Z);
        }

        /// <summary>
        /// Operator for subtracting two points.
        /// </summary>
        /// <param name="p1">The first point to subtract.</param>
        /// <param name="p2">The second point to subtract from.</param>
        /// <returns>Returns the difference between points.</returns>
        public static Point3f operator -(Point3f p1, Point3f p2)
        {
            return new Point3f(p1.X - p2.X, p1.Y - p2.Y, p1.Z - p2.Z);
        }

        /// <summary>
        /// Operation for scaling a point with a factor.
        /// </summary>
        /// <param name="p">The point to scale.</param>
        /// <param name="f">The factor to scale with.</param>
        /// <returns></returns>
        public static Point3f operator *(Point3f p, float f)
        {
            return new Point3f(p.X * f, p.Y * f, p.Z * f);
        }

        /// <summary>
        /// Gets the length of a 3D point from origo.
        /// </summary>
        /// <param name="p">The 3D point to receive the length from.</param>
        /// <returns>Returns the length of 3D point from origo.</returns>
        public static float Length(Point3f p)
        {
            return (float)(Math.Sqrt(p.X * p.X + p.Y * p.Y + p.Z * p.Z));
        }

        /// <summary>
        /// Normalizes a vector to units.
        /// </summary>
        /// <param name="p">The vector to normalize.</param>
        /// <returns>Returns the normalized vector.</returns>
        public static Point3f Normalize(Point3f p)
        {
            float length = (float)(Math.Sqrt(p.X * p.X + p.Y * p.Y + p.Z * p.Z));
            if (length == 0)
                return new Point3f(0, 0, 0);
            return new Point3f(p.X / length, p.Y / length, p.Z / length);
        }

        public static Point3f CrossProduct(Point3f p1, Point3f p2)
        {
            Point3f p = new Point3f();
            p.X = p1.Y * p2.Z - p1.Z * p2.Y;
            p.Y = p1.Z * p2.X - p1.X * p2.Z;
            p.Z = p1.X * p2.Y - p1.Y * p2.X;
            return p;
        }

        public static float DotProduct(Point3f p1, Point3f p2)
        {
            return p1.X * p2.X + p1.Y * p2.Y + p1.Z * p2.Z;
        }

        /// <summary>
        /// Gets the angle of 2D vector in radians.
        /// </summary>
        /// <param name="x">The x coordinate.</param>
        /// <param name="y">The y coordinate.</param>
        /// <returns>Returns the angle of 2D vector.</returns>
        public static float Angle(float x, float y)
        { 
            float alpha;
            if (x == 0)
                if (y > 0)
                    alpha = (float)(Math.PI / 2);
                else if (y < 0)
                    alpha = (float)(-Math.PI / 2);
                else
                    alpha = 0;
            else
            {
                alpha = (float)Math.Atan(y / x);
                if (x < 0 && y < 0)
                    alpha -= (float)Math.PI;
                else if (x < 0)
                    alpha += (float)Math.PI;
            }
            return alpha;
        }
    }
}
