using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace ShapeBlending
{
    class SimpleSlideBar : SimpleControl
    {
        private GuiLook m_look;
        private int m_lineHeight = 10;
        private int m_sliderWidth = 10;
        private ColorFadeEffect m_fadeLine;
        private ColorFadeEffect m_fadeBorder;
        private GlutWindow m_window;

        public event EventHandler ValueChanged;

        private float m_value = 0;

        private bool m_mouseDown = false;

        /// <summary>
        /// Gets or sets the slide bar value.
        /// </summary>
        public float Value
        {
            get { return m_value; }
            set { m_value = value; }
        }

        /// <summary>
        /// Compute a line rectangle.
        /// </summary>
        public Rectangle LineRectangle
        {
            get { return new Rectangle(0, (int)(m_size.Height / 2 - m_lineHeight / 2), m_size.Width - 1, m_lineHeight); }
        }

        /// <summary>
        /// Computes a slide rectangle.
        /// </summary>
        public Rectangle SlideRectangle
        {
            get 
            {
                int scrollWidth = m_size.Width - 1 - m_sliderWidth;
                return new Rectangle((int)(m_value * scrollWidth), 0, m_sliderWidth, m_size.Height - 1); 
            }
        }

        /// <summary>
        /// Constructor for the slide bar.
        /// </summary>
        /// <param name="look">The look of slide bar.</param>
        public SimpleSlideBar(GuiLook look)
        {
            m_look = look;
            m_fadeLine = new ColorFadeEffect(
                m_look.InactiveControlColor,
                m_look.ActiveControlColor, m_look.FadeInterval);
            m_fadeBorder = new ColorFadeEffect(
                m_look.InactiveBorderColor,
                m_look.ActiveBorderColor, m_look.FadeInterval);
            m_size = new Size(150, 40);
        }

        public override void Draw(System.Drawing.Graphics g)
        {
            // Draw background.
            int width = m_size.Width - 1;
            g.FillRectangle(new SolidBrush(m_fadeLine.GetColor()), LineRectangle);
            g.DrawRectangle(new Pen(m_fadeBorder.GetColor(), 1), LineRectangle);

            g.FillRectangle(new SolidBrush(m_fadeLine.GetColor()), SlideRectangle);
            g.DrawRectangle(new Pen(m_fadeBorder.GetColor(), 1), SlideRectangle);
        }

        public override bool CaptureMouse(int x, int y, GlutWindow window)
        {
            m_fadeLine.Start();
            m_fadeBorder.Start();
            m_window = window;
            m_window.MouseDown += new MouseEventHandler(MouseDown);
            m_window.MouseMove += new MouseEventHandler(MouseMove);
            m_window.MouseUp += new MouseEventHandler(MouseUp);
            return true;
        }

        public override bool ReleaseMouse(int x, int y)
        {
            if (m_mouseDown)
                return false;

            if (!(x >= m_position.X &&
                y >= m_position.Y &&
                x <= m_position.X + m_size.Width &&
                y <= m_position.Y + m_size.Height))
            {
                m_window.MouseDown -= new MouseEventHandler(MouseDown);
                m_window.MouseMove -= new MouseEventHandler(MouseMove);
                m_window.MouseUp -= new MouseEventHandler(MouseUp);
                m_window = null;
                m_fadeLine.Reset();
                m_fadeBorder.Reset();
                return true;
            }
            return false;
        }

        public void MouseDown(object sender, MouseEventArgs e)
        {
            m_mouseDown = true;    
            // Change the value of slide bar.
            m_value = (float)Math.Min(Math.Max((double)(e.X - m_position.X) / m_size.Width, 0), 1);
            if (ValueChanged != null)
                ValueChanged(this, new EventArgs());
        }

        public void MouseMove(object sender, MouseEventArgs e)
        {
            if (m_mouseDown)
            {
                // Change the value of slide bar.
                m_value = (float)Math.Min(Math.Max((double)(e.X - m_position.X) / m_size.Width, 0), 1);
                if (ValueChanged != null)
                    ValueChanged(this, new EventArgs());
            }
        }

        public void MouseUp(object sender, MouseEventArgs e)
        {
            m_mouseDown = false;
        }
    }
}
