using System;
using System.Collections.Generic;
using System.Text;

namespace ShapeBlending
{
    public class Vector3f
    {
        public float X;
        public float Y;
        public float Z;

        public Vector3f()
        {
            X = 0;
            Y = 0;
            Z = 0;
        }

        public Vector3f(float newX, float newY, float newZ)
        {
            X = newX;
            Y = newY;
            Z = newZ;
        }

        public Vector3f(float[] rhs)
        {
            X = rhs[0];
            Y = rhs[1];
            Z = rhs[2];
        }

        public Vector3f(Vector3f rhs)
        {
            X = rhs.X;
            Y = rhs.Y;
            Z = rhs.Z;
        }

        public void Set(float newX, float newY, float newZ)
        {
            X = newX;
            Y = newY;
            Z = newZ;
        }

        public void LoadZero()
        {
            X = 0;
            Y = 0;
            Z = 0;
        }

        public void LoadOne()
        {
            X = 1;
            Y = 1;
            Z = 1;
        }

        public Vector3f CrossProduct(Vector3f rhs)
        {
            return new Vector3f(Y * rhs.Z - Z * rhs.Y,
                Z * rhs.X - X * rhs.Z, X * rhs.Y - Y * rhs.X);
        }

        public float DotProduct(Vector3f rhs)
        {
            return X * rhs.X + Y * rhs.Y + Z * rhs.Z;
        }

        public void Normalize()
        {
            float length = GetLength();

            if (length == 1 || length == 0)			//return if length is 1 or 0
                return;

            float scalefactor = 1.0f / length;
            X *= scalefactor;
            Y *= scalefactor;
            Z *= scalefactor;
        }

        public Vector3f GetNormalized()
        {
            Vector3f result = new Vector3f(this);
            result.Normalize();
            return result;
        }

        public Vector3f GetRotatedX(double angle)
        {
            if (angle == 0.0)
                return this;

            float sinAngle = (float)Math.Sin(Math.PI * angle / 180);
            float cosAngle = (float)Math.Cos(Math.PI * angle / 180);

            return new Vector3f(X,
                                Y * cosAngle - Z * sinAngle,
                                Y * sinAngle + Z * cosAngle);
        }

        public Vector3f GetRotatedY(double angle)
        {
            if (angle == 0.0)
                return this;

            float sinAngle = (float)Math.Sin(Math.PI * angle / 180);
            float cosAngle = (float)Math.Cos(Math.PI * angle / 180);

            return new Vector3f(X * cosAngle + Z * sinAngle,
                                Y,
                                -X * sinAngle + Z * cosAngle);
        }

        public Vector3f GetRotatedZ(double angle)
        {
            if (angle == 0.0)
                return this;

            float sinAngle = (float)Math.Sin(Math.PI * angle / 180);
            float cosAngle = (float)Math.Cos(Math.PI * angle / 180);

            return new Vector3f(X * cosAngle - Y * sinAngle,
                            X * sinAngle + Y * cosAngle,
                            Z);
        }

        public Vector3f GetRotatedAxis(double angle, Vector3f axis)
        {
            if (angle == 0.0)
                return this;

            Vector3f u = axis.GetNormalized();

            Vector3f rotMatrixRow0 = new Vector3f();
            Vector3f rotMatrixRow1 = new Vector3f();
            Vector3f rotMatrixRow2 = new Vector3f();

            float sinAngle = (float)Math.Sin(Math.PI * angle / 180);
            float cosAngle = (float)Math.Cos(Math.PI * angle / 180);
            float oneMinusCosAngle = 1.0f - cosAngle;

            rotMatrixRow0.X = (u.X) * (u.X) + cosAngle * (1 - (u.X) * (u.X));
            rotMatrixRow0.Y = (u.X) * (u.Y) * (oneMinusCosAngle) - sinAngle * u.Z;
            rotMatrixRow0.Z = (u.X) * (u.Z) * (oneMinusCosAngle) + sinAngle * u.Y;

            rotMatrixRow1.X = (u.X) * (u.Y) * (oneMinusCosAngle) + sinAngle * u.Z;
            rotMatrixRow1.Y = (u.Y) * (u.Y) + cosAngle * (1 - (u.Y) * (u.Y));
            rotMatrixRow1.Z = (u.Y) * (u.Z) * (oneMinusCosAngle) - sinAngle * u.X;

            rotMatrixRow2.X = (u.X) * (u.Z) * (oneMinusCosAngle) - sinAngle * u.Y;
            rotMatrixRow2.Y = (u.Y) * (u.Z) * (oneMinusCosAngle) + sinAngle * u.X;
            rotMatrixRow2.Z = (u.Z) * (u.Z) + cosAngle * (1 - (u.Z) * (u.Z));

            return new Vector3f(this.DotProduct(rotMatrixRow0),
                                this.DotProduct(rotMatrixRow1),
                                this.DotProduct(rotMatrixRow2));
        }

        public Vector3f GetPackedTo01()
        { 
            Vector3f temp = new Vector3f(this);
	        temp.Normalize();
	        temp=temp*0.5f+new Vector3f(0.5f, 0.5f, 0.5f);
	        return temp;
        }

        public static Vector3f operator *(float scaleFactor, Vector3f rhs)
        {
            return rhs * scaleFactor;
        }

        public static Vector3f operator *(Vector3f rhs, float scaleFactor)
        {
            return new Vector3f(rhs.X * scaleFactor, rhs.Y * scaleFactor,
                rhs.Z * scaleFactor);
        }

        public new bool Equals(object o)
        {
            Vector3f rhs = (Vector3f)o;
            if (rhs.X == X && rhs.Y == Y && rhs.Z == Z)
                return true;
            return false;
        }

        public float GetLength()
        {
            return (float)Math.Sqrt((X * X) + (Y * Y) + (Z * Z));
        }

        public float GetSquaredLength()
        {
            return (X * X) + (Y * Y) + (Z * Z);
        }

        public Vector3f Lerp(Vector3f v2, float factor)
        {
            return this * (1.0f - factor) + v2 * factor;
        }

        public Vector3f QuadraticInterpolate(Vector3f v2, Vector3f v3, float factor)
        {
            return this * (1.0f - factor) * (1.0f - factor) + 2 * v2 * factor * (1.0f - factor) + v3 * factor * factor;
        }

        public static Vector3f operator +(Vector3f v, Vector3f rhs)
        {
            return new Vector3f(v.X + rhs.X, v.Y + rhs.Y, v.Z + rhs.Z);
        }

        public static Vector3f operator -(Vector3f v, Vector3f rhs)
        {
            return new Vector3f(v.X - rhs.X, v.Y - rhs.Y, v.Z - rhs.Z);
        }

        public static Vector3f operator *(Vector3f v, Vector3f rhs)
        {
            return new Vector3f(v.X * rhs.X, v.Y * rhs.Y, v.Z * rhs.Z);
        }

        public static Vector3f operator / (Vector3f v, float rhs)
        {
            return (rhs==0.0f) ? new Vector3f(0.0f, 0.0f, 0.0f) : new Vector3f(v.X / rhs, v.Y / rhs, v.Z / rhs);
        }

        /// <summary>
        /// Returns the minimum vector of two vectors.
        /// </summary>
        /// <param name="v1">The first vector to compare.</param>
        /// <param name="v2">The second vector to compare.</param>
        /// <returns>Returns the minimum vector.</returns>
        public static Vector3f Min(Vector3f v1, Vector3f v2)
        {
            return new Vector3f(Math.Min(v1.X, v2.X),
                Math.Min(v1.Y, v2.Y),
                Math.Min(v1.Z, v2.Z));
        }

        /// <summary>
        /// Returns the maximum vector of two vectors.
        /// </summary>
        /// <param name="v1">The first vector to compare.</param>
        /// <param name="v2">The second vector to compare.</param>
        /// <returns>Returns the maximum vector.</returns>
        public static Vector3f Max(Vector3f v1, Vector3f v2)
        {
            return new Vector3f(Math.Max(v1.X, v2.X),
                Math.Max(v1.Y, v2.Y),
                Math.Max(v1.Z, v2.Z));
        }
    }
}
